#!/usr/bin/env ruby

DBPATH = "/var/db/pkg/"
TIME_FMT = "%Y/%m/%d %H:%M:%S"
EVM_VER = "0.1.13"


class Evm

    attr_reader :recent_day, :regex_word, :sort_key, :show_columns,
                :show_package_size, :items, :reverse_sort

    def initialize

        @recent_day = Time.at(0)
        @regex_word = nil
        @sort_key = ""

        @show_package_size = nil
        @show_columns = Array.new
        @show_package_size = false
        @items = Array.new
        @reverse_sort = false

=begin
unless [].respond_to? :sort_by
    module Enumerable
        def sort_by
            self.collect { |i| [yield(i),i] }
            sort { |a,b| a[0] <=> b[0] }
            collect! { |i| i[1] }
        end
    end
=end

    end

    # =========================================================================
    # 
    # =========================================================================

    def set_show_columns( keys )

        return if @show_columns.size != 0

        keys.each_byte{ |x|
            case x.chr
            when "b"
                @show_columns.push( 'build_host' )
            when 'c'
                @show_columns.push( 'cflags' )
            when 'd'
                @show_columns.push( 'description' )
            when 'g'
                @show_columns.push( 'group' )
            when 'i'
                @show_columns.push( 'install_date' )
            when 'l'
                @show_columns.push( 'license' )
            when 'n'
                @show_columns.push( 'name' )
            when 'N'
                @show_columns.push( 'name_ver_rel' )
            when 'p'
                @show_columns.push( 'packager' )
            when 'r'
                @show_columns.push( 'release' )
            when 's'
                @show_columns.push( 'size' )
            when 't'
                @show_columns.push( 'slot' )
            when 'u'
                @show_columns.push( 'url' )
            when 'v'
                @show_columns.push( 'version' )
            else
                raise OptionError
                abort
            end
        }

    end

    # =========================================================================
    # 
    # =========================================================================

    def set_sort_key( key )

#        return if @sort_key.length > 0

        case key
        when "group", "name", "install_date"
            @sort_key = key
        else

# raise an Exception
            raise OptionError
            show_usage
            abort
        end

    end

    # =========================================================================
    # 
    # =========================================================================

    def set_name_search_with_RE( re, ignore_case )

        return if @regex_word
        @regex_word = Regexp.new( re, ignore_case )

    end

    # =========================================================================
    # 
    # =========================================================================

    def set_show_package_size

#       return if @show_package_size
        @show_package_size = true

    end

    # =========================================================================
    # 
    # =========================================================================

    def set_reverse_sort

#       return if @reverse_sort
        @reverse_sort = true

    end

    # =========================================================================
    # 
    # =========================================================================

    def get_db

        Dir.foreach(DBPATH){ |group|

            next if group == "." or group == ".."

            Dir.foreach( DBPATH+group ){ |dir_name|

                next if dir_name == "." or dir_name == ".."

                Dir.chdir( DBPATH+group+"/"+dir_name )

                install_date = File.mtime( Dir["USE"].to_s )

                next if install_date < @recent_day

                name, version, release = get_name_ver_rel( dir_name )

                next unless @regex_word =~ name if @regex_word

                data_arr = Array.new
                File.open( Dir['*.ebuild'].to_s, "r" ).each_line do |line|
#                    data_arr.push( line.chomp ) if /^DESCRIPTION|^SLOT|^LICENSE|^\#\s[$]Header:/ =~ line
#                    data_arr.push( line.chomp ) if /^DESC|^SLOT|^LICE|^HOMEP|^\#\ \$Hea/ =~ line
                    data_arr.push( line.chomp ) if /^DESC|^SLOT|^HOMEP|^\#\ \$Hea/ =~ line
                end
                description, slot, license, url, packager = pickout_data( data_arr )

                if slot.empty? then
                    File.open( Dir['SLOT'].to_s, "r" ).each_line { |line|
                        slot = line.chomp
                    }
                end

                if license.empty? then
                    File.open( Dir['LICENSE'].to_s, "r" ).each_line { |line|
                        license = line.chomp
                    }
                end

                bhost = ""
                File.open( Dir['CHOST'].to_s, "r" ).each_line { |line|
                    bhost = line.chomp
                }


                cflags = ""
                File.open( Dir['CFLAGS'].to_s, "r" ).each_line { |line|
                    cflags = line.chomp
                }

# It's too slow.
                if @show_package_size then
                    size = get_total_size( Dir['CONTENTS'].to_s )
                else
                    size = 0
                end

                @items.push({"name" => name, "version" => version, "release" => release,
                             "install_date" => install_date, "group" => group, "size" => size.to_s,
                             "url" => url, "name_ver_rel" => dir_name, "packager" => packager,
                             "slot" => slot, "build_host" => bhost, "cflags" => cflags,
                             "license" => license, "description" => description })
            }
        }

        return @items

    end

    # =========================================================================
    # 
    # =========================================================================

    def get_total_size( content )

        size = 0

        File.open( content ).each_line{ |x|

            begin
                size += File.size( (x.split)[1] ) if /^obj/ =~ x
            rescue
            end

        }

        return size

    end

    # =========================================================================
    # 
    # =========================================================================

    def get_name_ver_rel( x )

###
### 
###

        from = x.rindex("-")
        rel = x[ from+1 .. -1 ]

        if rel[0].chr != "r" then
            rel = ""
            from = x.length
        end

        to = from

        from = x.rindex( "-", to-1 ).to_i
        ver = x[from+1...to]

        return [ x[0...from], ver, rel ]

    end

    # =========================================================================
    # 
    # =========================================================================

    def pickout_data( data_arr )

        description = ""
        slot = ""
        license = ""
        packager = ""
        url = "(none)"

        data_arr.each{ |str|

            case str
            when /^DESC/
                description = str[ 13 .. -2 ]
            when /^SLOT/
                slot = str[ 5 .. -1 ]
                slot = slot[1 .. -2] if slot[0].chr == "\""
            when /^LICE/
                license = str[ 8 .. -1 ]
            when /^\#\ \$Hea/
               packager = str.split(" ")[6]
            when /^HOMEP/
               url = str[ 10 .. -2 ]
            end
        }

        return [ description, slot, license, url, packager ]

    end

    # =========================================================================
    # 
    # =========================================================================

    def sort_by_column

        return if @sort_key.empty?
        @items = @items.sort_by{ |x| x[@sort_key] }

        @items.reverse! if @reverse_sort

    end

    # =========================================================================
    # 
    # =========================================================================

    def set_latest_days( days )

        return if @recent_day > Time.at(0)

        tmp = Time.now - 60 * 60 * 24 * days.to_i
        @recent_day = Time.local( tmp.year, tmp.month, tmp.day )

    end

    # =========================================================================
    # 
    # =========================================================================

    def get_version
        EVM_VER
    end


end
