#!/usr/bin/env ruby

$:.unshift( File.dirname($0).untaint )
require "evm"
require "getoptlong"

EVR_VER="0.1.13"

class Evr

    def initialize

        @evm = Evm.new
        @htmlname = String.new
        @csvname = String.new
        @sep_line = String.new
        @path = Dir.getwd
        @argstr = ARGV.join(" ")

    end

    # ======================================================================
    # 
    # ======================================================================
    def check_options

        return if ARGV.empty?

        if ( ARGV.size == 1 ) and ( ARGV[0][0].chr != "-" ) then
            ARGV.unshift("-n")
        end

        opts = GetoptLong.new(
            ["--xhtml",				"-x",	GetoptLong::REQUIRED_ARGUMENT],
            ["--csv",				"-c",	GetoptLong::REQUIRED_ARGUMENT],
            ["--days",				"-d",	GetoptLong::REQUIRED_ARGUMENT],
            ["--name",				"-n",	GetoptLong::REQUIRED_ARGUMENT],
            ["--name-ignorecase",	"-N",	GetoptLong::REQUIRED_ARGUMENT],
            ["--sort",				"-s",	GetoptLong::REQUIRED_ARGUMENT],
            ["--separator",			"-p",	GetoptLong::REQUIRED_ARGUMENT],
            ["--items",				"-i",	GetoptLong::REQUIRED_ARGUMENT],
            ["--help",				"-h",	GetoptLong::NO_ARGUMENT],
            ["--version",			"-v",	GetoptLong::NO_ARGUMENT],
            ["--show-package-size",	"-z",	GetoptLong::NO_ARGUMENT],
            ["--reverse-sort",		"-r",	GetoptLong::NO_ARGUMENT]
        )

        begin
            opts.each { | opt, arg |

                case opt
                when "--version"
                    show_version
                    exit

                when "--help"
                    show_usage
                    exit

                when "--xhtml"
                    set_htmlname( arg )

                when "--csv"
                    set_csvname( arg )

                when "--sort"
                    set_sort_key2evm( arg )

                when "--separator"
                    set_separator( arg )

                when "--days"
                    set_latest_days2evm( arg )

                when "--name"
                    set_name_search_with_RE2evm( arg, nil )

                when "--name-ignorecase"
                    set_name_search_with_RE2evm( arg, true )

                when "--items"
                    set_show_columns2evm( arg )

                when "--reverse-sort"
                    set_reverse_sort2evm

                when "--show-package-size"
                    set_show_package_size2evm

                else
                    STDERR.print "[#{$0}] An error occured!!\n"
                    abort
                end
            }
        rescue
            exit
        end

    end

    # ======================================================================
    # 
    # ======================================================================

    def show_usage

        print <<EOU
Usage : evr.rb [options [file]]
 -x,\t--xhtml [filename]\t\tOutput data as XHTML file
 -c,\t--csv [filename]\t\tOutput data as CSV File
 -d,\t--days Numeric\t\t\tInstalled f\or [Numeric] latest day
 -v,\t--version\t\t\tShow Version
 -h,\t--help \t\t\t\tShow Usage
 -n,\t--name[regexp]\t\t\tSearch with regular expression
 -N,\t--name-ignorecase[regexp]\tSearch with regular expression( ignore Case )
 -s,\t--sort key\t\t\tSort by key
 -z,\t--show-pakage-size\t\tInstalled package size
\t[keys] date, name, group
 -i,\t--Items\t\t\t\tSpecify Items
\t[items]
\t\tb - Build Host    c - cflags         d - Description
\t\tg - Group         i - Install date   l - License        n - Name
\t\tN - Name-Version-Release
\t\tp - Packager      r - Release        s - Size
\t\tt - Slot          u - URL            v - Version
 -r,\t--reverse-sort\t\t\tShow data with reverse sort
 -p,\t--searator[charactor]\t\tSeparator is put in between each data
\tn - Do not use separator(default)
\te - Equal marks are used f\or separator
\th - Hyphens are used f\or separator
\tl - An empty line is put in
EOU

    end

    # ======================================================================
    # SHOW VERSION
    # EVR and EVM
    # ======================================================================

    def show_version

        print "EVR version #{EVR_VER}\n"
        print "EVM version " + @evm.get_version + "\n"
        print "This program may be freely redistributed under the terms of the GNU GPL v2\n"

    end

    # ======================================================================
    # Set OUTPUT HTML filename
    # ======================================================================

    def set_htmlname( html_filename )

        if @htmlname.length != 0 then
            STDERR.print "#{$0}: Warning : < #{html_filename} > was ignored. [-x/--xhtml] was already specified.\n"
            return
        end

        @htmlname = html_filename

    end

    # ======================================================================
    # Set Shown Columns
    # ======================================================================

    def set_show_columns2evm( columns )

        if @evm.show_columns.size != 0 then
            STDERR.print "#{$0}: Warning : < #{columns} > was ignored. [-i/--items] was already specified.\n"
            return
        end

        columns.each_byte{ |x|

            if /[bcdgilnNprstuv]/ =~ x.chr then
                next
            else
                STDERR.print "#{$0}: Error : Illegal Option in [-i] < #{x.chr} > \n"
                show_usage
                STDERR.print "#{$0}: Abort!!\n"
                abort
            end
        }

        @evm.set_show_columns( columns )

    end

    # ======================================================================
    # Set OUTPUT CSV filename
    # ======================================================================

    def set_csvname( csv_filename )

        if @csvname.length != 0 then
            STDERR.print "#{$0}: Warning : < #{csv_filename} > was ignored. [-c/--csv] was already specified.\n"
            return
        end

        @csvname = csv_filename

    end

    # ======================================================================
    # 
    # ======================================================================

    def set_separator( sep_char )

        if @sep_line.size != 0 then
            STDERR.print "#{$0}: Warning : < #{sep_char} > was ignored. [-p/--separator] was already specified.\n"
            return
        end

        case sep_char
        when 'n'
            return
        when 'h'
            @sep_line = ('-' * 80) + "\n"
        when 'e'
            @sep_line = ('=' * 80) + "\n"
        when 'l'
            @sep_line = "\n"
        else
            STDERR.print "#{$0}: Error : Illegal Option in [-p] < #{sep_char} > \n"
            STDERR.print "#{$0}: Abort!!\n"
            abort
        end

    end

    # ======================================================================
    # 
    # ======================================================================

    def set_latest_days2evm( days )

        if @evm.recent_day != Time.at(0) then
            STDERR.print "#{$0}: Warning : < #{days} > was ignored. [-d/--days] was already specified.\n"
            return
        end

        @evm.set_latest_days( days )

    end

    # ======================================================================
    # 
    # ======================================================================

    def set_name_search_with_RE2evm( re, ignore_case )

        if @evm.regex_word then
            STDERR.print "#{$0}: Warning : < #{re} > was ignored. [-n/-N/--name/--name-ignorecase] was already specified.\n"
            return
        end

        @evm.set_name_search_with_RE( re, ignore_case  )

    end

    # ======================================================================
    # 
    # ======================================================================

    def set_sort_key2evm( key )

        if @evm.sort_key.length > 0 then
            STDERR.print "#{$0}: Warning : < #{key} > was ignored. [-s/--sort] was already specified.\n"
        end

        case key
        when "date"
            @evm.set_sort_key( "install_date" )
        when "group", "name"
            @evm.set_sort_key( key )
        else

            STDERR.print "[#{$0}] - Illegal sortkey [ #{key} ]!!\n\n"
            show_usage
            abort

        end

    end

    # ======================================================================
    # 
    # ======================================================================

    def set_show_package_size2evm

        if @evm.show_package_size then
            STDERR.print "#{$0}: Warning : [-z/--show-package-size] was already specified.\n"
        end

        @evm.set_show_package_size

    end

    # ======================================================================
    # 
    # ======================================================================

    def set_reverse_sort2evm

        if @evm.reverse_sort then
            STDERR.print "#{$0}: Warning : [-r/--reverse-sort] was already specified.\n"
        end

        @evm.set_reverse_sort

    end

    # =========================================================================
    # 
    # =========================================================================

    def print_packages_orig

        @evm.items.each{ |i|

            @evm.show_columns.each{ |x|

                if i[x].instance_of? Time then
                    printf("%12-s : %s\n", x, i[x].strftime( TIME_FMT ))
                else
                    printf("%12-s : %s\n", x, i[x])
                end
            }

            print @sep_line
        }

    end

    # =========================================================================
    # 
    # =========================================================================

    def print_package_epm_format

        STDERR.printf("Just a moment...\n\n")

        @evm.items.each{ |x|

            printf("Name        : %s\n"\
                   "Version     : %s\n"\
                   "Release     : %28-s        Slot: %s\n"\
                   "Install date: %28-s  Build Host: %s\n"\
                   "Group       : %28-s     License: %s\n"\
                   "Size        : %s\n"\
                   "Packager    : %s\n"\
                   "URL         : %s\n"\
                   "Description :\n"\
                   "%s\n",
                    x["name"], x["version"], x["release"], x["slot"], x["install_date"], x["build_host"],
                    x["group"], x["license"], x["size"], x["packager"], x["url"], x["description"] )

            print @sep_line

        }

    end

    # =========================================================================
    # 
    # =========================================================================

    def print_packages

        if @evm.show_columns.size > 0 then
            print_packages_orig
        else
            print_package_epm_format
        end

    end

    # =========================================================================
    # 
    # =========================================================================

    def print_packages_count

        case @evm.items.size
        when 0
            print "No package.\n"
        when 1
            print "1 package.\n"
        else
            print @evm.items.size, " packages.\n"
        end

    end

    # =========================================================================
    # 
    # =========================================================================

    def output_csv
        Dir.chdir( @path )

        set_show_columns2evm("nvrigbptlsud") if @evm.show_columns.empty?

        f = File.new( @csvname, "w" )

        begin

            f.print @evm.show_columns.collect{ |x|
                "\"" + x + "\""
            }.join(",") + "\n"

            tArr = Array.new
            @evm.items.each{ |i|

                @evm.show_columns.each{ |c|

                    i[c] = i[c].strftime( TIME_FMT ) if i[c].instance_of? Time
                    tArr.push( "\"" + i[c].to_s + "\"" )

                }

                f.print tArr.join(",") + "\n"
                tArr.clear

            }

        rescue
            STDERR.print "#{$0} : ERROR!! Cannot output file!", "\n"
        ensure
            f.close
        end

    end

    # =========================================================================
    # 
    # =========================================================================

    def output_html

        Dir.chdir( @path )

        set_show_columns2evm("nvrigbptlsud") if @evm.show_columns.empty?
        f = File.new( @htmlname, "w" )

        begin

            f.print '<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
<head>
  <meta http-equiv="content-type" content="text/html;CHARSET=UTF-8" />
  <meta http-equiv="content-style-type" content="text/css" />
  <title>evr</title>
  <style type="text/css">
    body{ margin : 8px; padding : 8px; background-color : #e8ffe8; }
    th{ background-color : #addddd; }
    .name{ background-color : #beeeee; }
    .name_ver_rel{ background-color : #beeeee; }
    .install_date{ background-color : #bfffff; }
    .build_host{ background-color : #bfffff; }
    .description{ background-color : #bfffff; }
    .group{ background-color : #bfffff; }
    .license{ background-color : #bfffff; }
    .packager{ background-color : #bfffff; }
    .release{ background-color : #bfffff; }
    .size{ background-color : #bfffff; }
    .url{ background-color : #bfffff; }
    .slot{ background-color : #bfffff; }
    .version{ background-color : #bfffff; }
  </style>
</head>
<body>'

            f.print "<p>You exec [ <kbd>" + $0 + " " + @argstr + "</kbd>] at "
            f.print Time.now.strftime( TIME_FMT )
            f.print ".</p>\n"

            f.print "<p>EVR : " + EVR_VER + " | EVR : " + @evm.get_version + "</p>\n"

            f.print "<div align=\"center\">\n"
            f.print "<table width='92%' frame='border' rules='all' cellpadding='3'>\n"

            f.print "<tr>", @evm.show_columns.collect{ |x|
                "<th>" + x + "</th>"
            }, "</tr>\n"

            tArr = Array.new
            @evm.items.each{ |i|

                @evm.show_columns.each{ |c|
                    i[c] = i[c].strftime( TIME_FMT ) if i[c].instance_of? Time
                    i[c] = "<a href=\"#{i[c]}\">" + i[c] + "</a>" if /^http:|ftp:/ =~ i[c]
                    tArr.push( "<td class=\"#{c}\">" + i[c].to_s + "</td>" )
                }

                f.print "<tr>" + tArr.to_s + "</tr>" + "\n"
                tArr.clear

            }

            f.print "</table>\n"
            f.print "</div>\n"

            f.print "</body>\n"
            f.print "</html>"

        rescue
            STDERR.print "#{$0} : ERROR!! Cannot output file!", "\n"
        ensure
            f.close
        end

    end

    # ======================================================================
    # 
    # ======================================================================

    def execute

        check_options
        @evm.get_db
        @evm.sort_by_column

        if @csvname.empty? and @htmlname.empty? then

            print_packages
            print_packages_count
            return

        end

        if @csvname.length > 0
            output_csv
        end

        if @htmlname.length > 0
            output_html
        end

    end

end

# ==================

Evr.new.execute
